# -*- coding: utf-8 -*-


require 'date'
require 'sidfm_controller_support'

class SidfmLproductsController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private
  
  include SidfmControllerSupport
  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_roles

  before_action :get_vm_version
  before_action :clear_import_session, :except => [ :index, :read_import_file, :progress_import, :import ]
  before_action :clear_export_session, :except => [ :export ]
  before_action -> { get_summary_table_lines('lproducts') }

  def check_roles
    if !@sidfm_roles.include?(:admin) then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
  end

  def filter_lproducts
    searched_lproducts_name = @lproducts[:search_lproducts]
    searched_versions = @lproducts[:search_versions]
    searched_flg = @lproducts[:search_flg_enable]
    if searched_lproducts_name.size > 0 then
      @all_lproducts = @all_lproducts.where('LOWER(name) LIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, searched_lproducts_name.downcase)}%")
    end
    if searched_versions.size > 0 then
      p_ids = SidfmdLversion.where('LOWER(version) LIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, searched_versions.downcase)}%").map{|v| v.sidfmd_lproduct_id}
      @all_lproducts = @all_lproducts.where(:id => p_ids)
    end
    if searched_flg.size > 0 then
      @all_lproducts = @all_lproducts.where(:flg_enable => (searched_flg == 'enable' ? true : false))
     end
  end

  def sort_lproducts
    order_param = @lproducts[:order].split('_')
    col = order_param[0]
    order = order_param[1]
    if col == "lproducts-priority" then
      @all_lproducts = @all_lproducts.sort_by{|p| p.priority}
    elsif col == "lproducts-name" then
      @all_lproducts = @all_lproducts.sort_by{|p| p.name.downcase}
    elsif col == "lproducts-status" then
      if order == "asc" then
        @all_lproducts = @all_lproducts.sort{|a, b| 
          r = (a.flg_enable ? 100: 1000) <=> (b.flg_enable ? 100: 1000)
          if r == 0 then
            r = a.priority <=> b.priority
          end
          r
        }
      else
        @all_lproducts = @all_lproducts.sort{|a, b|
          r = (a.flg_enable ? 1000: 100) <=> (b.flg_enable ? 1000: 100)
          if r == 0 then
            r = a.priority <=> b.priority
          end
          r
        }
      end
    else #どれにも当てはまらない場合は、priority順にする。(普通の操作では起きない)
      @all_lproducts = @all_lproducts.sort_by{|p| p.priority}
    end
    @all_lproducts = @all_lproducts.reverse if order == "desc" && col != "lproducts-status"
  end

  def create_content
    content = []
    create_versions_data = ->(product_versions){
      edit_id = 1;
      versions = []
      product_versions.sort_by{|v| v.priority}.each do |v|
        versions << {
          :id => v.id,
          :flg_enable => v.flg_enable,
          :fullname => v.fullname,
          :priority => v.priority,
          :version => v.version,
          :edit_id => edit_id, #バージョンを編集する際に使用するid
          :release_date => (v.release_date ? SidfmdLproduct.date_to_string(v.release_date) : nil),
          :support_date => (v.support_date ? SidfmdLproduct.date_to_string(v.support_date) : nil),
          :eol_date => (v.eol_date ? SidfmdLproduct.date_to_string(v.eol_date) : nil)
        }
        edit_id += 1  
      end 
      return versions
    }  

    @all_lproducts.each do |lproduct|
      content << { 
        :product => lproduct, 
        :versions => create_versions_data.(lproduct.sidfmd_lversions)
      }
    end
    content
  end

  def strip_file_item_values(item)
    item.each do |key, val|
      if val && val.is_a?(String) then
        item[key] = val.strip
      end
    end
  end

  def get_date(str_date)
    date = { :valid => true, :date => '' } 
    if str_date and str_date.size > 0 then
      begin
        date[:date] = Date.parse(str_date)
      rescue Exception => e
        date[:valid] = false
      end
    else
      date[:date] = nil
    end
    date
  end

  public 

  def index
    f_params = params.fetch(:pages, {}).permit(:offset, :per_page, :order, :filters, :checked, :all_checked, :clear_session)
    if f_params[:clear_session] then
      clear_import_session
      redirect_to :action => :index
    end
    @lproducts = {}
    @lproducts[:per_page] = (@summary_table_lines || f_params[:per_page] || 10).to_i
    @lproducts[:offset] = (f_params[:offset] || 0).to_i
    @lproducts[:filters] = JSON.parse(f_params[:filters] || '{}').symbolize_keys
    @lproducts[:checked] = JSON.parse(f_params[:checked] || '[]')
    @lproducts[:all_checked] = ((f_params[:all_checked] || '') == 'true')
    @lproducts[:order] = (f_params[:order] || 'lproducts-priority_asc')
    @lproducts[:search_lproducts] = @lproducts[:filters][:search_lproducts] || ''
    @lproducts[:search_versions] = @lproducts[:filters][:search_versions] || ''
    @lproducts[:search_flg_enable] = @lproducts[:filters][:search_flg_enable] || ''
    #全プロダクト
    @all_lproducts = SidfmdLproduct.all
 
    #表示するプロダクト
    filter_lproducts #検索
    sort_lproducts #ソート
    @all_lproduct_ids = []
    @all_lproduct_names_with_id = {}
    @all_lproducts.each do |p|
      @all_lproduct_ids << p.id
      @all_lproduct_names_with_id[p.id] = p.name
    end
    @lproducts[:total_size] = @all_lproducts.size
    @all_lproducts = @all_lproducts.slice(@lproducts[:offset]..(@lproducts[:offset] + @lproducts[:per_page] - 1))#@all_lproducts.offset(@lproducts[:offset]).limit(@lproducts[:per_page]) 
    @all_lproducts = [] unless @all_lproducts
    @lproducts[:content] = create_content #フィルター後のデータ
    
    @enable_flgs = {
      '有効' => 'enable',
      '無効' => 'disable'
    }

    #インポートエラーがあればエラー内容を表示する
    @import_notices = session[:import_notices]
    clear_import_session

    #ブラウザバックかどうか判断するための処理
    #if !session['chkbok_ids'] then
    #  session['chkbok_ids'] = {}
    #end
    #if !session['chkbok_ids']['lproducts'] then
    #  session['chkbok_ids']['lproducts'] = {}
    #end
    #session['chkbok_ids']['lproducts']['is_browser_back'] = false
  end

  def create
    f_params = params.fetch(:form, {}).permit(:lprod_name, :lprod_priority, :eol_date, :lprod_status, :versions)
    dump = {}
    prod_name = f_params[:lprod_name]
    priority = f_params[:lprod_priority]
    flg_enable = f_params[:lprod_status]
    #同じ名前のプロダクトは登録できないようにする
    if SidfmdLproduct.find_by(:name => prod_name) then
       redirect_to :action => :index, :notice => '既に登録済みのプロダクトです。' and return
    end
    begin 
      versions = JSON.parse(f_params[:versions])
    rescue JSON::ParserError => e
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    lprod_eol_date = get_date(f_params[:eol_date])
    if !lprod_eol_date[:valid] then    
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    if !prod_name || prod_name.size == 0 || !priority || !flg_enable then  
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    priority = priority.to_i
    dump[:action] = :index
    SidfmdLproduct.transaction do 
      begin 
        new_prod = SidfmdLproduct.new(
          flg_enable: flg_enable == 'enable' ? true : false,
          name: prod_name,
          priority: priority,
          eol_date: lprod_eol_date[:date]
        )
        f_versions = []
        is_pass = true
        versions.each do |version|
          version = version.symbolize_keys
          release_date = get_date(version[:release_date])
          support_date = get_date(version[:support_date])
          eol_date = get_date(version[:eol_date])
          [ release_date, support_date, eol_date ].each do |date|
            if !date[:valid] then
              is_pass = false
              break
            end
          end
          f_versions << SidfmdLversion.new(
            flg_enable: version[:flg_enable],
            version: version[:version],
            fullname: new_prod.name + ' ' + version[:version],
            priority: version[:priority],
            release_date: release_date[:date],
            support_date: support_date[:date],
            eol_date: eol_date[:date]
          )
        end
        if !is_pass then
          redirect_to :action => :index, :notice => '不正な操作です。' and return
        end
        new_prod.sidfmd_lversions = f_versions
        if new_prod.save then
          #プロダクトの順位整列
          SidfmdLproduct.where('priority >= ?', priority).where.not(:id => new_prod.id).each do |p|
            p.update_column(:priority, p.priority + 1)
          end
        end
        Rails.cache.clear
      rescue Exception => e
        dump[:notice] = 'プロダクトの登録に失敗しました。'
        redirect_to dump and return
      end
    end
    redirect_to dump 
  end

  def modify
    f_params = params.fetch(:form, {}).permit(:lprod_name, :lprod_priority, :lprod_status, :eol_date, :versions, :lproduct_id, :offset, :per_page, :order, :filters)
    prod_id = f_params[:lproduct_id].to_i
    prod_name = f_params[:lprod_name]
    priority = f_params[:lprod_priority]
    flg_enable = f_params[:lprod_status]
    dump = {}
    dump[:action] = :index
    #同じ名前のプロダクトは登録できないようにする
    same_name_prod = SidfmdLproduct.find_by(:name => prod_name)
    if same_name_prod && same_name_prod.id != prod_id then
      redirect_to :action => :index, :notice => '既に登録済みのプロダクトです。' and return
    end
    begin 
      versions = JSON.parse(f_params[:versions])
    rescue JSON::ParseError => e
      redirect_to :action => :index, :notice => '不正な操作です。' and return 
    end
    if !prod_name && prod_name.size == 0 || !priority || !flg_enable then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    lprod_eol_date = get_date(f_params[:eol_date])
    if !lprod_eol_date[:valid] then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    begin 
      SidfmdLproduct.transaction do
        is_pass = true
        f_versions = []
        versions.each do |version|
          version = version.symbolize_keys
          release_date = get_date(version[:release_date])
          support_date = get_date(version[:support_date])
          eol_date = get_date(version[:eol_date])
          [ release_date, support_date, eol_date ].each do |date|
            if !date[:valid] then
              is_pass = false
              break
            end
          end
          version[:release_date] = release_date[:date]
          version[:support_date] = support_date[:date]
          version[:eol_date] = eol_date[:date]
          f_versions << version  
        end
        if !is_pass then
          redirect_to :action => :index, :notice => '不正な操作です。' and return
        end
        versions = f_versions
        priority = priority.to_i
        prod = SidfmdLproduct.find_by_id(prod_id)
        if prod then
          old_prod_name = prod.name
          old_priority = prod.priority
          prod.update_attributes(
            name: prod_name,
            flg_enable:flg_enable == 'enable' ? true : false,
            priority: priority,
            eol_date: lprod_eol_date[:date]
          )
          #バージョン更新
          current_versions = prod.sidfmd_lversions
          version_ids = versions.map{|v| v["id"]}
          del_versions = current_versions.where.not(:id => version_ids)
          del_version_ids = del_versions.pluck(:id)
          del_versions.destroy_all
          SidfmdLocalVulLapp.where(:sidfmd_lversion_id => del_version_ids).destroy_all
          SidfmdHostLapp.where(:sidfmd_lversion_id => del_version_ids).destroy_all
          versions.each do |v|
            target_version = current_versions.find_by_id(v[:id])
            if !target_version then
              SidfmdLversion.create(
                sidfmd_lproduct_id: prod.id, 
                flg_enable: v[:flg_enable],
                version: v[:version],
                fullname: prod.name + ' ' + v[:version],
                priority: v[:priority],
                release_date: v[:release_date],
                support_date: v[:support_date],
                eol_date: v[:eol_date]
              )
            else
              if old_prod_name != prod.name ||
                 target_version.flg_enable != v[:flg_enable] ||
                 target_version.version != v[:version] ||
                 target_version.priority != v[:priority] ||
                 target_version.release_date != v[:release_date] ||
                 target_version.support_date != v[:support_date] ||
                 target_version.eol_date != v[:eol_date]
              then 
                target_version.update_attributes(
                  flg_enable: v[:flg_enable],
                  version: v[:version],
                  fullname: prod.name + ' ' + v[:version],
                  priority: v[:priority],
                  release_date: v[:release_date],
                  support_date: v[:support_date],
                  eol_date: v[:eol_date]
                )
              end  
            end 
          end 
          #プロダクトの順位整列
          if old_priority != prod.priority then
            if prod.priority > old_priority then
              SidfmdLproduct.where('priority > ?', old_priority)
                            .where('priority <= ?', prod.priority)
                            .where.not(:id => prod.id)
                            .each do |p|
                p.update_attribute(:priority, p.priority - 1)
              end
            else
              SidfmdLproduct.where('priority < ?', old_priority)
                            .where('priority >= ?', prod.priority)
                            .where.not(:id => prod.id)
                            .each do |p|
                p.update_attribute(:priority, p.priority + 1)
              end
            end
          end
        end
      end
      Rails.cache.clear
      dump[:'pages[offset]'] = f_params[:offset].to_i
      dump[:'pages[per_page]'] = f_params[:per_page]
      dump[:'pages[filters]'] = f_params[:filters] || '{}'
      dump[:'pages[order]'] = f_params[:order]
    rescue Exception => e
      logger.info(e.inspect)
      redirect_to :action => :index, :notice => 'バージョンの更新に失敗しました。' and return
    end 
    redirect_to dump
  end

  def delete
    f_params = params.fetch(:form, {}).permit(:delete_id)
    delete_id = f_params[:delete_id]
    if !delete_id then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    begin
      SidfmdLproduct.transaction do
        prod = SidfmdLproduct.find_by_id(delete_id)
        if prod then
          prod_priority = prod.priority
          if SidfmdLversion.where(:sidfmd_lproduct_id => delete_id).destroy_all && prod.destroy then
            #プロダクトの順位整列
            SidfmdLproduct.where('priority > ?', prod_priority).each do |p|
              p.update_column(:priority, p.priority - 1)
            end
            #ロカール脆弱性の関連削除
            SidfmdLocalVulLapp.where(:sidfmd_lproduct_id => delete_id).destroy_all
            #ホストの関連削除
            SidfmdHostLapp.where(:sidfmd_lproduct_id => delete_id).destroy_all
          end
        end
      end
    rescue Exception => e
      redirect_to :action => :index, :notice => 'プロダクトの削除に失敗しました。' and return
    end
    Rails.cache.clear
    redirect_to :action => :index
  end

  def bulk_lproduct_status
    f_params = params.fetch(:form, {}).permit(:lproduct_ids, :flg_enable, :offset, :per_page, :order, :filters)
    dump ={}
    begin 
      prod_ids = JSON.parse(f_params[:lproduct_ids])
    rescue JSON::ParseError => e
      redirect_to :action => :index, :notice => '不正な操作です。' and return 
    end
    flg_enable = f_params[:flg_enable]
    if !flg_enable then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    begin 
      SidfmdLproduct.transaction do 
        SidfmdLproduct.where(:id => prod_ids).update_all(:flg_enable => flg_enable == 'enable' ? true : false)
      end
      Rails.cache.clear
      dump[:action] = :index
      dump[:'pages[offset]'] = f_params[:offset]
      dump[:'pages[per_page]'] = f_params[:per_page]
      dump[:'pages[filters]'] = f_params[:filters] || '{}'
      dump[:'pages[order]'] = f_params[:order]
    rescue Exception => e
      logger.info(e.inspect)
      redirect_to :action => :index, :notice => '更新に失敗しました。' and return
    end 
    redirect_to dump
  end

  def bulk_lproducts_delete
    f_params = params.fetch(:form, {}).permit(:delete_ids) 
    begin
      delete_ids = JSON.parse(f_params[:delete_ids])
    rescue JSON::ParseError => e
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    begin 
      SidfmdLproduct.transaction do
        SidfmdLproduct.where(:id => delete_ids).destroy_all
        SidfmdLversion.where(:sidfmd_lproduct_id => delete_ids).destroy_all 
        #プロダクトの順位整列
        priority = 1
        SidfmdLproduct.all.order(:priority).each do |p|
          if p.priority != priority then
            p.update_column(:priority, priority)
          end
          priority += 1
        end
        #ロカール脆弱性の関連削除
        SidfmdLocalVulLapp.where(:sidfmd_lproduct_id => delete_ids).destroy_all
        #ホストの関連削除
        SidfmdHostLapp.where(:sidfmd_lproduct_id => delete_ids).destroy_all
      end
      Rails.cache.clear
    rescue Exception => e
      logger.info(e.inspect)
      redirect_to :action => :index, :notice => 'プロダクトの削除に失敗しました。' and return
    end
    redirect_to :action => :index
  end

  CSV_BOM = "\xff\xfe".force_encoding('UTF-16LE').freeze

  #import
  
  def read_import_file
    f_params = params.permit(:lproduct_file, :lversion_file)
    lproduct_file = f_params[:lproduct_file]
    lversion_file = f_params[:lversion_file]
    is_err = false
    r = {}
    begin
      if lproduct_file then
        clear_import_session#先にローカルプロダクト情報ファイルを読み込む。sessionをリセット。
        import_lversions = {}
        import_lversions[:not_new] = {}
        import_lversions[:new] = {}
        session[:import_lversions] = import_lversions
        session[:import_notices] = []
        session[:import_actions] = []
        lproduct_file = lproduct_file.read
        if lproduct_file.getbyte(0) == 255 and lproduct_file.getbyte(1) == 254 then
          lproduct_file = lproduct_file[2, lproduct_file.length - 2]
        end
        lproduct_file = lproduct_file.force_encoding('UTF-16LE').encode('UTF-8')
        csv = CSV.parse(lproduct_file, :col_sep => "\t", :headers => :first_row)
        import_lproducts = []
        new_lproduct_ids = []
        priorities = []
        lproduct_ids = []
        lproduct_names = []
        cnt = 1
        csv.each do |line|
          cnt += 1
          item = {}
          item[:new_product_id] = line['プロダクト登録ID']
          item[:priority] = line['優先順位']
          item[:product_id] = line['プロダクトID']
          item[:name] = line['プロダクト名']
          item[:eol_date] = line[SidfmdLproduct::EOL_DATE_LABEL]
          item[:flg_enable] = line['有効無効']
          item[:flg_delete] = line['削除']
          item[:line_num] = cnt
          strip_file_item_values(item)
          import_lproducts << item
        
          same_new_product_id_idx = new_lproduct_ids.index(item[:new_product_id])
          same_priority_idx = priorities.index(item[:priority])
          same_id_idx = lproduct_ids.index(item[:product_id])
          same_name_idx = lproduct_names.index(item[:name])
    
          if item[:new_product_id] && item[:new_product_id].size > 0 && item[:product_id] && item[:product_id].size > 0 then
            session[:import_notices] << "ローカルプロダクト情報ファイルの#{cnt}行目: プロダクト登録IDとプロダクトIDはどちらか一方のみ入力して下さい。"
          end
          if same_new_product_id_idx && item[:new_product_id] && item[:new_product_id].size > 0 then
            session[:import_notices] << "ローカルプロダクト情報ファイルの#{same_new_product_id_idx + 2}行目と#{cnt}行目: プロダクト登録IDが重複しています。"
          end
          if same_priority_idx && item[:priority] && item[:priority].size > 0 && item[:flg_delete] && item[:flg_delete].downcase == 'false'then
            session[:import_notices] << "ローカルプロダクト情報ファイルの#{same_priority_idx + 2}行目と#{cnt}行目: 優先順位が重複しています。"
          end
          if same_id_idx && item[:product_id] && item[:product_id].size > 0 then
            session[:import_notices] << "ローカルプロダクト情報ファイルの#{same_id_idx + 2}行目と#{cnt}行目: プロダクトIDが重複しています。"
          end
          if same_name_idx && item[:name] && item[:name].size > 0 then
            session[:import_notices] << "ローカルプロダクト情報ファイルの#{same_name_idx + 2}行目と#{cnt}行目: プロダクト名が重複しています。"
          end
          if !item[:priority] || item[:priority].size == 0 && item[:flg_delete] && item[:flg_delete].downcase == 'false' then
            session[:import_notices] << "ローカルプロダクト情報ファイルの#{cnt}行目: 優先順位が入力されておりません。"
          end
          if !item[:name] || item[:name].size == 0 then
            session[:import_notices] << "ローカルプロダクト情報ファイルの#{cnt}行目: プロダクト名が入力されておりません。"
          end
          new_lproduct_ids << item[:new_product_id]
          priorities << item[:priority]
          lproduct_ids << item[:product_id]
          lproduct_names << item[:name]
        end
        if import_lproducts.size == 0 then
          session[:import_notices] << "ローカルプロダクト情報ファイルに有効なデータがありません。"
        else
          session[:import_lproducts] = import_lproducts
          r[:total_size] = import_lproducts.size
        end
        if session[:import_notices].size > 0 then
          is_err = true
        end
      elsif lversion_file then
        import_lversions = {}
        import_lversions[:not_new] = {}
        import_lversions[:new] = {}
        lversion_file = lversion_file.read
        if lversion_file.getbyte(0) == 255 and lversion_file.getbyte(1) == 254 then
          lversion_file = lversion_file[2, lversion_file.length - 2]
        end
        lversion_file = lversion_file.force_encoding('UTF-16LE').encode('UTF-8')
        csv = CSV.parse(lversion_file, :col_sep => "\t", :headers => :first_row)
        version_ids = []
        cnt = 1
        csv.each do |line|
          cnt += 1
          item = {}
          item[:new_product_id] = line['プロダクト登録ID']
          product_id = (line['プロダクトID'].is_a?(String) ? line['プロダクトID'].strip : line['プロダクトID'])
          item[:product_id] = product_id
          item[:priority] = line['優先順位']
          item[:version_id] = line['バージョンID']
          item[:version] = line['バージョン名']
          item[:release_date] = line[SidfmdLproduct::RELEASE_DATE_LABEL]
          item[:support_date] = line[SidfmdLproduct::SUPPORT_DATE_LABEL]
          item[:eol_date] = line[SidfmdLproduct::EOL_DATE_LABEL]
          item[:flg_enable] = line['有効無効']
          item[:flg_delete] = line['削除']
          item[:line_num] = cnt
          strip_file_item_values(item)
      
          same_version_id_line_num = version_ids.index(item[:version_id])
          if item[:new_product_id] && item[:new_product_id].size > 0 && item[:product_id] && item[:product_id].size > 0 then
            session[:import_notices] << "ローカルバージョン情報ファイルの#{cnt}行目: プロダクト登録IDとプロダクトIDはどちらか一方のみ入力して下さい。"
            version_ids << item[:version_id]
            next
          end
          if same_version_id_line_num && item[:version_id] && item[:version_id].size > 0 then
            session[:import_notices] << "ローカルバージョン情報ファイルの#{same_version_id_line_num + 2}行目と#{cnt}行目: バージョンIDが重複しています。"
            version_ids << item[:version_id]
            next
          end
          if item[:new_product_id] && item[:new_product_id].size > 0 then
            import_lversions[:new][item[:new_product_id]] = [] unless import_lversions[:new][item[:new_product_id]]
            import_lversions[:new][item[:new_product_id]] << item
          elsif item[:product_id] && item[:product_id].size > 0 then
            import_lversions[:not_new][product_id] = [] unless import_lversions[:not_new][product_id]
            import_lversions[:not_new][product_id] << item
          else
            session[:import_notices] << "ローカルバージョン情報ファイルの#{cnt}行目: プロダクト登録IDとプロダクトIDの両方とも入力されていません。"         
          end
          version_ids << item[:version_id]
        end
        session[:import_lversions] = import_lversions
      else
        is_err = true
      end
    rescue Exception => e
      log_exception(e)
      is_err = true
    end
    r[:err] = is_err
    render :layout => false, :json => r
  end

  def progress_import
    import_lversions = session[:import_lversions]
    session[:import_notices] = [] unless session[:import_notices]
    actions = session[:import_actions]
    r = {}
    if session[:import_lproducts].size > 0 then
      lproducts = session[:import_lproducts].shift(30)
      executed_size = lproducts.size
      lproducts.each do |lprod|
        action = {}
        line_num = lprod['line_num']
        lprod_id = lprod['product_id']
        new_lprod_id = lprod['new_product_id']
        #入力チェック
        if lprod['flg_enable'] && lprod['flg_enable'].size > 0 && 
           lprod['flg_enable'].downcase != 'true' && lprod['flg_enable'].downcase != 'false' 
        then
          session[:import_notices] << "ローカルプロダクト情報ファイルの#{line_num}行目: 有効無効は「TRUE」もしくは「FALSE」を入力して下さい。"
          next
        end
        lprod['flg_enable'] = lprod['flg_enable'].downcase if lprod['flg_enable']
        if lprod['flg_delete'] && lprod['flg_delete'].size > 0 && lprod['flg_delete'].downcase != 'true' then
          session[:import_notices] << "ローカルプロダクト情報ファイルの#{line_num}行目: 削除は「TRUE」のみ有効です。"
          next
        end
        lprod['flg_delete'] = lprod['flg_delete'].downcase if lprod['flg_delete']
        if (lprod['priority'] =~ /^[0-9０-９]+$/) == nil then
          session[:import_notices] << "ローカルプロダクト情報ファイルの#{line_num}行目: 優先順位は正数で入力して下さい。"
          next
        end
        lprod['priority'] = lprod['priority'].tr('０-９','0-9').to_i
        if lprod['priority'] == 0 then
          session[:import_notices] << "ローカルプロダクト情報ファイルの#{line_num}行目: 優先順位は正数で入力して下さい。"
          next
        end

        eol_date = get_date(lprod['eol_date'])
        if !eol_date[:valid] then
          session[:import_notices] << "ローカルプロダクト情報ファイルの#{line_num}行目: #{SidfmdLproduct::EOL_DATE_LABEL}は日付を入力して下さい。"
          next
        end
        lprod['eol_date'] = eol_date[:date]

        if lprod_id && lprod_id.size > 0 then
          #既存のプロダクト
          if (lprod_id =~ /^[0-9０-９]+$/) == nil then
            session[:import_notices] << "ローカルプロダクト情報ファイルの#{line_num}行目: プロダクトIDは数字で入力して下さい。"
            next
          end
          lprod['product_id'] = lprod['product_id'].tr('０-９','0-9')
          registered_lprod = SidfmdLproduct.find_by(:id => lprod['product_id'])
          if registered_lprod then
            options = []
            if lprod['flg_delete'] && lprod['flg_delete'].size > 0 && lprod['flg_delete'].downcase == 'true' then
              lprod['eol_date'] = (lprod['eol_date'] ? SidfmdLproduct.date_to_string(lprod['eol_date']) : '')
              action = {
                :action => 'delete',
                :lprod => lprod,
                :product_name => registered_lprod.name,
                :action_name => '削除'
              }
              actions << action
              next
            end
            if registered_lprod.flg_enable.to_s != lprod['flg_enable'].downcase then
              options << {
                :action => 'flg_enable_change',
                :new_data => lprod['flg_enable'].downcase,
                :old_data => registered_lprod.flg_enable.to_s,
                :action_name => '有効無効変更'
              }
            end
            if registered_lprod.name != lprod['name'] then
              options << {
                :action => 'product_name_change',
                :new_data => lprod['name'],
                :old_data => registered_lprod.name,
                :action_name => 'プロダクト名変更'
              }
            end
            if registered_lprod.priority != lprod['priority'].to_i then
              options << {
                :action => 'priority_change',
                :new_data => lprod['priority'],
                :old_data => registered_lprod.priority,
                :action_name => '優先順位変更'
              }
            end
            if registered_lprod.eol_date != lprod['eol_date'] then
               options << {
                :action => 'eol_date_change',
                :new_data => (lprod['eol_date'] ? SidfmdLproduct.date_to_string(lprod['eol_date']) : ''),
                :old_data => (registered_lprod.eol_date ? SidfmdLproduct.date_to_string(registered_lprod.eol_date) : ''),
                :action_name => "#{SidfmdLproduct::EOL_DATE_LABEL}変更"
              }
            end
            if options.any? then
              action = {
                :action => 'modify',
                :product_name => lprod['name'],
                :lprod => lprod,
                :options => options,
                :is_modify_prod => true,
                :action_name => '変更'
              }
            end
          else
            session[:import_notices] << "ローカルプロダクト情報ファイルの#{line_num}行目: プロダクトが存在しません。"
            next
          end
          lversions = import_lversions['not_new'][lprod_id]
        else
          #新規登録プロダクト
          if new_lprod_id && new_lprod_id.size > 0 && (new_lprod_id =~ /^[0-9０-９]+$/) == nil then
            session[:import_notices] << "ローカルプロダクト情報ファイルの#{line_num}行目: プロダクト登録IDは数字で入力して下さい。"
            next
          end
          lprod['new_product_id'] = lprod['new_product_id'].tr('０-９','0-9') if new_lprod_id
          lprod['eol_date'] = (lprod['eol_date'] ? SidfmdLproduct.date_to_string(lprod['eol_date']) : '')
          action = {
            :action => 'new',
            :product_name => lprod['name'],
            :lprod => lprod,
            :action_name => '新規登録'
          }
          if import_lversions['new'][new_lprod_id] && import_lversions['new'][new_lprod_id].size > 0 then
            lversions = import_lversions['new'][new_lprod_id]
          else
            lversions = nil
          end
        end
        lversions = [] unless lversions
        #バージョンのチェック
        #重複チェック
        version_names = {}  
        filtered_lversions = {}
        new_lversions = []
        err = false
        lversions.each do |lver|
          same_version_name_line_num = version_names[lver['version']]
          line_num = lver['line_num']
          if lver['flg_delete'] && lver['flg_delete'].size > 0 && 
             lver['flg_delete'].downcase != 'false' &&
             lver['flg_delete'].downcase != 'true' then
            session[:import_notices] << "ローカルバージョン情報ファイルの#{line_num}行目: 削除は「TRUE」のみ有効です。"
            err = true
          end
          lver['flg_delete'] = lver['flg_delete'].downcase if lver['flg_delete']
          if lver['flg_delete'] && lver['flg_delete'] == 'true' then
            if !lver['version_id'] then
              session[:import_notices] << "ローカルバージョン情報ファイルの#{line_num}行目: バージョンIDが入力されておりません。"
              err = true
            else
              if lver['version_id'] && lver['version_id'].size > 0 && (lver['version_id'] =~ /^[0-9０-９]+$/) == nil then
                session[:import_notices] << "ローカルバージョン情報ファイルの#{line_num}行目: バージョンIDは数字で入力して下さい。"
                err = true
              else
                lver['version_id'] = lver['version_id'].tr('０-９','0-9').to_i if lver['version_id'] && lver['version_id'].size > 0
                filtered_lversions[lver['version_id']] = lver
              end
            end 
          end

          if !lver['flg_delete'] || (lver['flg_delete'].size > 0 && lver['flg_delete'].downcase == 'false') then #削除するバージョンはチェックしない
            if !lver['priority'] || lver['priority'].size == 0 then
              session[:import_notices] << "ローカルバージョン情報ファイルの#{line_num}行目: 優先順位が入力されておりません。"
              err = true
            elsif lver['priority'] && lver['priority'].size > 0 && (lver['priority'] =~ /^[0-9０-９]+$/) == nil then
              session[:import_notices] << "ローカルバージョン情報ファイルの#{line_num}行目: 優先順位は正数で入力して下さい。"
              err = true
            else
              lver['priority'] = lver['priority'].tr('０-９','0-9').to_i
            end
            if lver['priority'] == 0 then
              session[:import_notices] << "ローカルバージョン情報ファイルの#{line_num}行目: 優先順位は正数で入力して下さい。"
              next
            end
            if same_version_name_line_num && lver['version'] && lver['version'].size > 0 then
              session[:import_notices] << "ローカルバージョン情報ファイルの#{same_version_name_line_num}行目と#{line_num}行目: バージョン名が重複しています。"
              err = true
            end
            if !lver['version'] || lver['version'].size == 0 then
              session[:import_notices] << "ローカルバージョン情報ファイルの#{line_num}行目: バージョン名が入力されておりません。"
              err = true
            end
            if lver['version_id'] && lver['version_id'].size > 0 && (lver['version_id'] =~ /^[0-9０-９]+$/) == nil then
              session[:import_notices] << "ローカルバージョン情報ファイルの#{line_num}行目: バージョンIDは数字で入力して下さい。"
              err = true
            end
          
            lv_release_date = get_date(lver['release_date'])
            if !lv_release_date[:valid] then
              session[:import_notices] << "ローカルバージョン情報ファイルの#{line_num}行目: #{SidfmdLproduct::RELEASE_DATE_LABEL}は日付を入力して下さい。"
              err = true
            end
            lver['release_date'] = lv_release_date[:date]
            
            lv_support_date = get_date(lver['support_date'])
            if !lv_support_date[:valid] then
              session[:import_notices] << "ローカルバージョン情報ファイルの#{line_num}行目: #{SidfmdLproduct::SUPPORT_DATE_LABEL}は日付を入力して下さい。"
              err = true
            end
            lver['support_date'] = lv_support_date[:date]

            lv_eol_date = get_date(lver['eol_date'])
            if !lv_eol_date[:valid] then
              session[:import_notices] << "ローカルバージョン情報ファイルの#{line_num}行目: #{SidfmdLproduct::EOL_DATE_LABEL}は日付を入力して下さい。"
              err = true
            end
            lver['eol_date'] = lv_eol_date[:date]

            lver['version_id'] = lver['version_id'].tr('０-９','0-9') if lver['version_id'] && lver['version_id'].size > 0
            if lver['flg_enable'] && lver['flg_enable'].size > 0 && 
               lver['flg_enable'].downcase != 'true' && lver['flg_enable'].downcase != 'false' then
              session[:import_notices] << "ローカルバージョン情報ファイルの#{line_num}行目: 有効無効は「TRUE」もしくは「FALSE」を入力して下さい。"
              err = true
            end
            lver['flg_enable'] = lver['flg_enable'].downcase if lver['flg_enable']
      
            if !err then
              if lver['version_id'] && lver['version_id'].size > 0 then
                lver['version_id'] = lver['version_id'].to_i
                filtered_lversions[lver['version_id']] = lver
              else
                new_lversions << lver
              end
            end
          end
          version_names[lver['version']] = line_num
        end
     
        #バージョンが存在するかチェック
        filtered_lversions.each do |lv_id, data|
          if !SidfmdLversion.where(:id => lv_id).exists? then
            session[:import_notices] << "ローカルバージョン情報ファイルの#{data['line_num']}行目: バージョンが存在しません。"
            err = true
          end
        end

        next if err #エラーがあれば、そのプロダクトは変更しない 
        #バージョンの変更をチェック
        new_filtered_lversions = []
        num = 1
        if lprod['product_id'] && lprod['product_id'].size > 0 then
          target_lversions = SidfmdLversion.where(:sidfmd_lproduct_id => lprod['product_id'])
          is_modify = false
          is_priority_change = false
          hit_version_ids = []
          unuse_version_names = []
          use_version_names = []
          target_lversions.each do |lv|
            lver_data = filtered_lversions[lv.id]
            if lver_data then
              hit_version_ids << lv.id
              if lver_data['flg_delete'] && lver_data['flg_delete'] == 'true' then
                is_modify = true
                unuse_version_names << lv.version
              else
                new_filtered_lversions << lver_data
                #バージョンに変更があるかチェック
                if (lver_data['flg_enable'] && lver_data['flg_enable'] != lv.flg_enable.to_s) ||
                   (lver_data['version'] != lv.version && (lver_data['version'] + '.0') != lv.version) ||
                   (lver_data['priority'] != lv.priority) || 
                   lver_data['release_date'] != lv.release_date ||
                   lver_data['support_date'] != lv.support_date ||
                   lver_data['eol_date'] != lv.eol_date then
                  is_modify = true
                  is_priority_change = true if lver_data['priority'] != lv.priority
                end
                #ファイルにないバージョン名で重複しないかチェックの準備
                if (lver_data['version'] != lv.version && (lver_data['version'] + '.0') != lv.version) then
                  unuse_version_names << lv.version
                  use_version_names << lver_data['version']
                end
                lver_data['release_date'] = (lver_data['release_date'] ? SidfmdLproduct.date_to_string(lver_data['release_date']) : '')
                lver_data['support_date'] = (lver_data['support_date'] ? SidfmdLproduct.date_to_string(lver_data['support_date']) : '')
                lver_data['eol_date'] = (lver_data['eol_date'] ? SidfmdLproduct.date_to_string(lver_data['eol_date']) : '')
              end
            else
              new_filtered_lversions << {
                'product_id' => lv.sidfmd_lproduct_id,
                'flg_enable' => lv.flg_enable.to_s,
                'version' => lv.version,
                'version_id' => lv.id,
                'priority' => lv.priority,
                'release_date' => (lv.release_date ? SidfmdLproduct.date_to_string(lv.release_date) : ''),
                'support_date' => (lv.support_date ? SidfmdLproduct.date_to_string(lv.support_date) : ''),
                'eol_date' => (lv.eol_date ? SidfmdLproduct.date_to_string(lv.eol_date) : '')
              }
            end
          end
          
          #ファイルにないバージョン名で重複しないかチェック
          target_lversions.where.not(:version => unuse_version_names).where(:version => (use_version_names + new_lversions.map{|new_lv| new_lv['version']})).each do |lv|
            if version_names[lv.version] then
              #他でエラーになっていると、ここには入らない
              session[:import_notices] << "ローカルバージョン情報ファイルの#{version_names[lv.version]}行目: 既存のバージョンと同じバージョンは登録できません。"
              err = true
            end
          end

          temp_new_lversions = []
          new_lversions.each do |v|
            v['release_date'] = (v['release_date'] ? SidfmdLproduct.date_to_string(v['release_date']) : '')
            v['support_date'] = (v['support_date'] ? SidfmdLproduct.date_to_string(v['support_date']) : '')
            v['eol_date'] = (v['eol_date'] ? SidfmdLproduct.date_to_string(v['eol_date']) : '')
            temp_new_lversions << v
          end
          new_lversions = temp_new_lversions

          new_filtered_lversions = new_filtered_lversions + new_lversions

          if !err && (is_modify || target_lversions.size != new_filtered_lversions.size) then
            new_filtered_lversions = new_filtered_lversions.sort{|a, b|
              ret = a['priority'] <=> b['priority']
            }
            #priotiryが同じだったら、ファイルにある方を優先する
            if new_filtered_lversions.size >= 2 then
              for n in 0..(new_filtered_lversions.size - 2) do
                v = new_filtered_lversions[n]
                v2 = new_filtered_lversions[n + 1]
                p = v['priority']
                p2 = v2['priority']
                if p == p2 then
                  if (filtered_lversions[v['version_id']] && !filtered_lversions[v2['version_id']]) || (!v['version_id'] && v2['version_id']) then
                    new_filtered_lversions[n + 1]['priority'] = p + 1
                  elsif !filtered_lversions[v['version_id']] && filtered_lversions[v2['version_id']] || (v['version_id'] && !v2['version_id']) then
                    new_filtered_lversions[n]['priority'] = p + 1
                  end
                end
              end
            end
            #優先順位の整合性を取って、数字がずれていないことを保証する(ずれていたら修正する処理)
            new_filtered_lversions = new_filtered_lversions.sort{|a, b|
              ret = a['priority'] <=> b['priority']
              if ret == 0 then
                ret = a['version'] <=> b['version']
              end
              ret
            }
            lver_priority = {}
            new_filtered_lversions.each do |lv|
              lv['priority'] = num
              lver_priority[lv['version_id']] = lv['priority']
              num += 1
            end
            #ファイルで誤って優先順が付けれている場合、整合性をとる処理で優先順を整えた前だと,
            #実際に変更がない場合でも変更されていることになってしまうため、再度整合性を取った後に優先順位の変更をチェック
            is_modify = true
            if target_lversions.size == new_filtered_lversions.size && is_priority_change then
              is_change = false
              target_lversions.each do |lv|
                if lver_priority[lv.id] != lv.priority then
                  is_change = true
                  break
                end
              end
              is_modify = is_change
            end
            if is_modify then
              if action.size == 0 then
                action = {
                  :action => 'modify',
                  :product_name => lprod['name'],
                  :lprod => lprod,
                  :new_lvers => new_filtered_lversions,
                  :old_lvers => target_lversions.map{|lv| 
                    {
                      :product_id => lv.sidfmd_lproduct_id,
                      :flg_enable => lv.flg_enable.to_s,
                      :version => lv.version,
                      :priority => lv.priority,
                      :release_date => (lv.release_date ? SidfmdLproduct.date_to_string(lv.release_date) : ''),
                      :support_date => (lv.support_date ? SidfmdLproduct.date_to_string(lv.support_date) : ''),
                      :eol_date => (lv.eol_date ? SidfmdLproduct.date_to_string(lv.eol_date) : '')
                    }
                  }.sort_by{|lv| lv[:priority]},
                  :is_modify_versions => true,
                  :action_name => '変更'
                }
              else
                action[:new_lvers] = new_filtered_lversions
                action[:old_lvers] = target_lversions.map{|lv|
                  {
                    :product_id => lv.sidfmd_lproduct_id,
                    :flg_enable => lv.flg_enable.to_s,
                    :version => lv.version,
                    :priority => lv.priority,
                    :release_date => (lv.release_date ? SidfmdLproduct.date_to_string(lv.release_date) : ''),
                    :support_date => (lv.support_date ? SidfmdLproduct.date_to_string(lv.support_date) : ''),
                    :eol_date => (lv.eol_date ? SidfmdLproduct.date_to_string(lv.eol_date) : '')
                  }
                }.sort_by{|lv| lv[:priority]}
                action[:is_modify_versions] = true
              end
            end
          end
        else
          #新規登録プロダクトのバージョン
          if new_lversions.any? then
            temp = []
            new_lversions.sort_by{|lver| lver['priority']}.each do |lver|
              #優先順の数字がずれていないことを保証する(ずれていたら修正する処理)
              lver['priority'] = num
              lver['release_date'] = (lver['release_date'] ? SidfmdLproduct.date_to_string(lver['release_date']) : '')
              lver['support_date'] = (lver['support_date'] ? SidfmdLproduct.date_to_string(lver['support_date']) : '')
              lver['eol_date'] = (lver['eol_date'] ? SidfmdLproduct.date_to_string(lver['eol_date']) : '')
              temp << lver
              num += 1
            end
            action[:new_lvers] = temp
          end
        end
        
        if action[:action] then
          action[:line_num] = line_num
          actions << action
        end
      end
      session[:import_actions] = actions
    else
      executed_size = 0
      #プロダクト名が既存のプロダクトと重複していないかチェック
      unuse_prod_names = []
      use_prod_names = {}
      session[:import_actions].each do |action|
        action = action.deep_symbolize_keys
        case action[:action]
        when 'delete' then
          unuse_prod_names << action[:product_name]
        when 'modify' then
          if action[:options] then
            action[:options].each do |op|
              if op[:action] == 'product_name_change' then
                unuse_prod_names << op[:old_data]
                use_prod_names[op[:new_data]] = action[:line_num]
              end
            end
          end
        when 'new' then
          use_prod_names[action[:product_name]] = action[:line_num]
        end
      end
      SidfmdLproduct.where.not(:name => unuse_prod_names).where(:name => use_prod_names.keys).each do |lp|
        session[:import_notices] << "ローカルプロダクト情報ファイルの#{use_prod_names[lp.name]}行目: 既存のプロダクト名と重複するプロダクト名は登録できません。" 
        r[:same_prod_name_err] = true
      end
    end
    r[:executed_size] = executed_size
    render :layout => false, :json => r
  end
 
  def import
    @notices = session[:import_notices]
    @actions = session[:import_actions]
    if !@actions then
      redirect_to :action => :index, :notice => 'インポートに失敗しました。' and return
    elsif @actions.empty? then
      redirect_to :action => :index, :notice => '変更がありません。' and return
    end
  end

  #インポート確認画面で「実行」した時にDBに変更内容を反映する
  def import_apply
    f_params = params.permit(:action_data)
    err_message = ''
    r = {}
    action = JSON.parse(f_params[:action_data], :symbolize_names => true)
    case action[:action]
    when 'new' then
      #新規登録
      new_prod = action[:lprod]
      prod_name = new_prod[:name]
      priority = new_prod[:priority].to_i
      ordered_all_lproducts = SidfmdLproduct.order(:priority)
      same_name_prod = ordered_all_lproducts.find_by(:name => prod_name)
      if same_name_prod then
        #同名のプロダクトは登録できないようにする。
        err_message = "「#{prod_name}」は同名のプロダクトが既に登録されているため、登録されませんでした。"
      else
        #priorityの整合性をとる
        #インポート確認時とインポート反映時でDBのデータが変わる可能性があるため、このimport_applyで整合性をとる
        max_priority_product = ordered_all_lproducts.last
        if max_priority_product then
          max_priority = max_priority_product.priority
          if priority > (max_priority + 1) then
            priority = max_priority + 1
          end
        end
      end
      if err_message.size == 0 then
        begin
          SidfmdLproduct.transaction do
            prod = SidfmdLproduct.new(
              flg_enable: new_prod[:flg_enable].to_s == 'false' ? false : true,
              name: new_prod[:name],
              priority: priority,
              eol_date: new_prod[:eol_date].to_date 
            )
            if action[:new_lvers] then
              versions = action[:new_lvers]
              versions = versions.map{|version|
                SidfmdLversion.new(
                  flg_enable: version[:flg_enable].to_s == 'false' ? false : true,
                  version: version[:version],
                  fullname: prod.name + ' ' + version[:version],
                  priority: version[:priority].to_i,
                  release_date: version[:release_date].to_date,
                  support_date: version[:support_date].to_date,
                  eol_date: version[:eol_date].to_date
                )     
              }
              prod.sidfmd_lversions = versions
            end
            prod.save
            #優先順位整列
            SidfmdLproduct.where('priority >= ?', priority).where.not(:id => prod.id).each do |p|
              p.update_column(:priority, p.priority + 1)
            end
          end 
        rescue Exception => e
          log_exception(e)
          err_message = "「#{prod_name}」の登録に失敗しました。"
        end
      end
    when 'modify' then
      #変更
      mod_prod = action[:lprod]
      prod_name = mod_prod[:name]
      begin
        SidfmdLproduct.transaction do 
          prod = SidfmdLproduct.find_by(:id => mod_prod[:product_id])
          old_prod_name = prod.name
          if prod then
            if action[:is_modify_prod] then
              #プロダクトの変更
              action[:options].each do |option|
                new_data = option[:new_data]
                case option[:action]
                when 'flg_enable_change' then
                  prod.update(:flg_enable => (new_data.to_s == 'false' ? false : true))
                when 'product_name_change' then
                  prod.update(:name => new_data)
                when 'eol_date_change' then
                  prod.update(:eol_date => new_data.to_date)
                when 'priority_change' then
                  old_priority = prod.priority
                  new_priority = new_data.to_i
                  ordered_all_lproducts = SidfmdLproduct.order(:priority)
                  max_priority = ordered_all_lproducts.last.priority
                  prod.update(:priority => new_priority)
                  #順位整列
                  if old_priority != new_priority then
                    if new_priority > old_priority then
                     SidfmdLproduct.where('priority > ?', old_priority)
                                    .where('priority <= ?', new_priority)
                                    .where.not(:id => prod.id)
                                    .each do |p|
                                      p.update_attribute(:priority, p.priority - 1)
                                    end
                    else
                      SidfmdLproduct.where('priority < ?', old_priority)
                                    .where('priority >= ?', new_priority)
                                    .where.not(:id => prod.id)
                                    .each do |p|
                                      p.update_attribute(:priority, p.priority + 1)
                                    end
                    end
                  end
                  if new_priority > (max_priority + 1) then
                    prod.update(:priority => max_priority)
                  end
                end 
              end
            end
            if action[:is_modify_versions] then
              #バージョン更新
              current_versions = prod.sidfmd_lversions
              new_versions = action[:new_lvers]
              version_ids = new_versions.map{|v| v[:version_id]}
              del_versions = current_versions.where.not(:id => version_ids)
              del_version_ids = del_versions.pluck(:id)
              del_versions.destroy_all
              SidfmdLocalVulLapp.where(:sidfmd_lversion_id => del_version_ids).destroy_all
              SidfmdHostLapp.where(:sidfmd_lversion_id => del_version_ids).destroy_all
              new_versions.each do |v|
                v = v.symbolize_keys
                v[:flg_enable] = v[:flg_enable].to_s == 'false' ? false : true
                target_version = current_versions.find_by_id(v[:version_id])
                if !target_version then
                  SidfmdLversion.create(
                    sidfmd_lproduct_id: prod.id,
                    flg_enable: v[:flg_enable],
                    version: v[:version],
                    fullname: prod.name + ' ' + v[:version],
                    priority: v[:priority],
                    release_date: v[:release_date].to_date,
                    support_date: v[:support_date].to_date,
                    eol_date: v[:eol_date].to_date
                  )
                else
                  if old_prod_name != prod.name ||
                    target_version.flg_enable != v[:flg_enable] ||
                    target_version.version != v[:version] ||
                    target_version.priority != v[:priority].to_i ||
                    target_version.release_date != v[:release_date].to_date ||
                    target_version.support_date != v[:support_date].to_date ||
                    target_version.eol_date != v[:eol_date].to_date
                  then
                    target_version.update_attributes(
                      flg_enable: v[:flg_enable],
                      version: v[:version],
                      fullname: prod.name + ' ' + v[:version],
                      priority: v[:priority],
                      release_date: v[:release_date].to_date,
                      support_date: v[:support_date].to_date,
                      eol_date: v[:eol_date].to_date
                    )
                  end
                end
              end
            end
          else
            err_message = "「#{prod_name}」は削除されているため、変更できませんでした。"
          end
        end
      rescue Exception => e
        log_exception(e)
        err_message = "「#{prod_name}」の登録に失敗しました。"
      end
    when 'delete'
      #削除
      delete_id = action[:lprod][:product_id]
      begin
        SidfmdLproduct.transaction do
          prod = SidfmdLproduct.find_by(:id => delete_id)
          if prod then
            prod_priority = prod.priority
            if SidfmdLversion.where(:sidfmd_lproduct_id => delete_id).destroy_all && prod.destroy then
              #プロダクトの順位整列
              SidfmdLproduct.where('priority > ?', prod_priority).each do |p|
                p.update_column(:priority, p.priority - 1)
              end
              #ロカール脆弱性の関連削除
              SidfmdLocalVulLapp.where(:sidfmd_lproduct_id => delete_id).destroy_all
              #ホストの関連削除
              SidfmdHostLapp.where(:sidfmd_lproduct_id => delete_id).destroy_all
            end          
          end
        end
      rescue Exception => e
        log_exception(e)
        err_message = "「#{prod_name}」の登録に失敗しました。"
      end
    end
    Rails.cache.clear
    r[:err_message] = err_message
  
    render :layout => false, :json => r
  end

  #export
  def export
    f_params = params.permit(:checked, :process, :executed_size, :target)
    process = f_params[:process]
    target = f_params[:target]
    err = false
    message = ''
    r = {}
    if !target || (target != 'lproducts' && target != 'lversions') then
      err = true
      messagei = '不正な操作です。'
      r[:err] = err
      r[:message] = message
      render :layout => false, :json => r and return
    end
    if process == 'start_export' then
      target_data = []
      begin
        checked = JSON.parse(f_params[:checked])
      rescue JSON::ParserError => e
        log_exception(e)
        err = true
        message = '不正な操作です。'
        render :layout => false, :json => r and return
      end
      if checked.size > 0 then
        if target == 'lproducts' then
          target_data = SidfmdLproduct.where(:id => checked).order(:priority)
        else
          target_data = SidfmdLversion.joins(:sidfmd_lproduct)
                                      .where('sidfmd_lversions.sidfmd_lproduct_id IN (?)', checked)
                                      .order('sidfmd_lproducts.priority')
                                      .order('sidfmd_lproducts.id')
                                      .order('sidfmd_lversions.priority')
                                      .select(
                                        'sidfmd_lversions.sidfmd_lproduct_id',
                                        'sidfmd_lversions.priority',
                                        'sidfmd_lversions.id',
                                        'sidfmd_lversions.version',
                                        'sidfmd_lversions.flg_enable',
                                        'sidfmd_lversions.release_date',
                                        'sidfmd_lversions.support_date',
                                        'sidfmd_lversions.eol_date'
                                      )
        end
      else
        if target == 'lproducts' then
          target_data = SidfmdLproduct.order(:priority)
        else
          target_data = SidfmdLversion.joins(:sidfmd_lproduct)
                                      .order('sidfmd_lproducts.priority')
                                      .order('sidfmd_lproducts.id')
                                      .order('sidfmd_lversions.priority')
                                      .select(
                                        'sidfmd_lversions.sidfmd_lproduct_id',
                                        'sidfmd_lversions.priority',
                                        'sidfmd_lversions.id',
                                        'sidfmd_lversions.version',
                                        'sidfmd_lversions.release_date',
                                        'sidfmd_lversions.support_date',
                                        'sidfmd_lversions.eol_date',
                                        'sidfmd_lversions.flg_enable'
                                      )
        end
      end
      clear_export_session
      session[:export_lproducts] = target_data
      r[:total_size] = target_data.size
      r[:err] = err
      r[:message] = message
      render :layout => false, :json => r
    elsif process == 'progress_export' then
      target_data = session[:export_lproducts]
      if !target_data then
        err = true
        message = 'データの取得に失敗しました。'
      else
        begin
          executed_size = f_params[:executed_size].to_i
          t_data = target_data.slice(executed_size, 50)
          t_data = [] unless t_data
          executed_size = t_data.size
          session[:export_lproduct_lines] = [] unless session[:export_lproduct_lines]
          lines = session[:export_lproduct_lines]
          if target == 'lproducts' then
            t_data.each do |lproduct|
              lines << [
                '',
                lproduct['priority'],
                lproduct['id'],
                lproduct['name'],
                (lproduct['eol_date'] ? SidfmdLproduct.date_to_string(lproduct['eol_date'].to_date) : ''),
                lproduct['flg_enable'],
                ''
              ]
            end
          else
            t_data.each do |lversion|
              lines << [
                '',
                lversion['sidfmd_lproduct_id'],
                lversion['priority'],
                lversion['id'],
                lversion['version'],
                (lversion['release_date'] ? SidfmdLproduct.date_to_string(lversion['release_date'].to_date) : ''),
                (lversion['support_date'] ? SidfmdLproduct.date_to_string(lversion['support_date'].to_date) : ''),
                (lversion['eol_date'] ? SidfmdLproduct.date_to_string(lversion['eol_date'].to_date) : ''),
                lversion['flg_enable'],
                ''
              ]
            end
          end
          session[:export_lproduct_lines] = lines
          r[:executed_size] = executed_size
        rescue Exception => e
          log_exception(e)
          err = true
          message = 'データの処理に失敗しました。'
        end
      end
      r[:err] = err
      r[:message] = message
      render :layout => false, :json => r
    else
      target_lines = session[:export_lproduct_lines]
      if !target_lines then
        err = true
        message = 'データの取得に失敗しました。'
      else
        if target == 'lproducts' then
          header = [
            'プロダクト登録ID'.encode('UTF-16LE'),
            '優先順位'.encode('UTF-16LE'),
            'プロダクトID'.encode('UTF-16LE'),
            'プロダクト名'.encode('UTF-16LE'),
            SidfmdLproduct::EOL_DATE_LABEL.encode('UTF-16LE'),
            '有効無効'.encode('UTF-16LE'),
            '削除'.encode('UTF-16LE')
          ]
          file_name = "vm-local-product-#{Time.now.strftime('%Y%m%d%H%M%S')}.csv"
        else
          header = [
            'プロダクト登録ID'.encode('UTF-16LE'),
            'プロダクトID'.encode('UTF-16LE'),
            '優先順位'.encode('UTF-16LE'),
            'バージョンID'.encode('UTF-16LE'),
            'バージョン名'.encode('UTF-16LE'),
            SidfmdLproduct::RELEASE_DATE_LABEL.encode('UTF-16LE'),
            SidfmdLproduct::SUPPORT_DATE_LABEL.encode('UTF-16LE'),
            SidfmdLproduct::EOL_DATE_LABEL.encode('UTF-16LE'),
            '有効無効'.encode('UTF-16LE'),
            '削除'.encode('UTF-16LE')
          ]
          file_name = "vm-local-product-versions-#{Time.now.strftime('%Y%m%d%H%M%S')}.csv"
        end
        csv = CSV.generate("#{CSV_BOM}", :col_sep => "\t".encode('UTF-16LE')) do |o|
          o << header
          target_lines.each do |line|
            o << line.map{|item| item.to_s.encode('UTF-16LE')}
          end 
        end
        send_data(csv, :filename => file_name, :disposition => 'attachment')
        cookies['export_flg'] = true
        clear_export_session
      end
    end
  end

  #api

  #登録時に同じプロダクト名が登録されていないかチェック
  def api_check_is_same_lprod_name
    f_params = params.permit(:method_name, :prod_name, :prod_id)
    method = f_params[:method_name]
    prod_name = f_params[:prod_name]
    prod_id = f_params[:prod_id].to_i
    is_same_prod_name = false
    if method == "modify" then
      same_name_prod = SidfmdLproduct.find_by(:name => prod_name)
      is_same_prod_name = true if same_name_prod && prod_id != same_name_prod.id
    else
      is_same_prod_name = true if SidfmdLproduct.find_by(:name => prod_name)
    end
    render :layout => false, :json => is_same_prod_name
  end

  #Issueに紐づくローカルプロダクト Issueのステータスが対策済み・影響無は除く
  def get_issue_related_lprod_ids
    f_params = params.permit(:lprod_ids)
    lprod_ids = JSON.parse(f_params[:lprod_ids])
    issue_related_lprod_ids = Issue.joins(:sidfmd_issue)
                                   .joins(
                                     'LEFT OUTER JOIN sidfmd_local_vuls ON sidfmd_local_vuls.html_id = sidfmd_issues.html_id'
                                   )
                                   .joins(
                                     'LEFT OUTER JOIN sidfmd_local_vul_lapps ON sidfmd_local_vuls.id = sidfmd_local_vul_lapps.sidfmd_local_vul_id'
                                   )
                                   .where.not('issues.status_id IN (?)', [ SidfmdIssue::STATUS_FIXED, SidfmdIssue::STATUS_NOTAFFECT ])
                                   .where('sidfmd_local_vul_lapps.sidfmd_lproduct_id IN (?)', lprod_ids)
                                   .select('sidfmd_local_vul_lapps.sidfmd_lproduct_id AS lpid')
                                   .map{|i| i.lpid}.uniq

    render :layout => false, :json => issue_related_lprod_ids 
  end

  #変更の時にIssueに紐づくローカルプロダクトのバージョンのうち、そのバージョンが削除されるプロダクトのidを返す
  def get_issue_related_remove_version_prod_ids
    f_params = params.permit(:lprod_ids_with_version_ids)
    lprod_ids_with_version_ids = JSON.parse(f_params[:lprod_ids_with_version_ids])
    lprod_ids = lprod_ids_with_version_ids.keys
    issue_related_lprod_ids_vesion_ids = {}
    Issue.joins(:sidfmd_issue)
         .joins(
           'LEFT OUTER JOIN sidfmd_local_vuls ON sidfmd_local_vuls.html_id = sidfmd_issues.html_id'
         )
         .joins(
           'LEFT OUTER JOIN sidfmd_local_vul_lapps ON sidfmd_local_vuls.id = sidfmd_local_vul_lapps.sidfmd_local_vul_id'
         )
         .where.not('issues.status_id IN (?)', [ SidfmdIssue::STATUS_FIXED, SidfmdIssue::STATUS_NOTAFFECT ])
         .where('sidfmd_local_vul_lapps.sidfmd_lproduct_id IN (?)', lprod_ids)
         .select('sidfmd_local_vul_lapps.sidfmd_lproduct_id AS lpid') 
         .select('sidfmd_local_vul_lapps.sidfmd_lversion_id AS lvid')
         .each do |lapp|
           if lapp.lvid && lapp.lvid != 0 then
             issue_related_lprod_ids_vesion_ids[lapp.lpid] = [] unless issue_related_lprod_ids_vesion_ids[lapp.lpid]
             issue_related_lprod_ids_vesion_ids[lapp.lpid] << lapp.lvid
           end
         end
    remove_prod_ids = []
    lprod_ids_with_version_ids.each do |lprod_id, lver_ids|
      lver_ids = lver_ids.map{|lv_id| lv_id.to_i}
      issue_related_lver_ids = issue_related_lprod_ids_vesion_ids[lprod_id.to_i]
      if issue_related_lver_ids && (issue_related_lver_ids - lver_ids).size > 0 then
        remove_prod_ids << lprod_id.to_i
      end
    end
    render :layout => false, :json => remove_prod_ids
  end
end
