
var finder_box = (function(){
  "use strict";
  var finder_box = {};
  var _this = finder_box;

  _this.init = function(){
    _this.j_input_text = $("#finder .j_input_text");
    _this.bind_event();
    var tmpl = $("#find_products_template").html();
    _this.compiled_template = _.template(tmpl);
  };

  _this.render_find_products = function(){
    $("#find_products").html(_this.compiled_template({products:_this.products}));
  };

  _this.bind_event = function(){
    _this.j_input_text.on("keyup", function(){_this.keyup_input_text(this)});
  };

  _this.keyup_input_text = function(elm){
    if(typeof _this.timer_id === "number"){
      clearTimeout(_this.timer_id);
    }
    _this.timer_id = setTimeout(function(){_this.keyup_timeout_input_text(elm)}, 500);
  };

  _this.keyup_timeout_input_text = function(elm){
    _this.timer_id = null;
    var input_value = _this.j_input_text.val().trim();
    if(_this.input_value !== input_value && input_value.length > 0){
      _this.input_value = input_value;
      var products_req = 
        $.post($('#sidfm_internal_api_info_find_products_path').val(), {search_text:input_value}, 
          function(res){_this.products = res.products}
        );
      $.when(products_req).done(function(){
        _this.render_find_products();
      });
    }
  };

  return _this;
})();

var new_products_box = (function(){
  "use strict";
  var new_products_box = {};
  var _this = new_products_box;

  _this.show = function(){
    _this.dt2 = new Date();
    _this.dt1 = new Date();
    _this.dt1.setMonth(_this.dt1.getMonth() - 3);
    var products_req = 
      $.post($('#sidfm_internal_api_info_products_path').val(), {begin_datetime:_this.dt1, end_datetime:_this.dt2, datetime_type:"create"}, 
        function(res){_this.products = res.products}
      );
    $.when(products_req).done(function(){
      _this.make_data();
      _this.render();
    });
  };

  _this.make_data = function(){
    var date_products_map = {};
    _this.products.forEach(function(product){
      product.versions.forEach(function(ver){
        if(ver.create_datetime != null){
          var d = ver.create_datetime.substr(0,10).replace(/-/g, "/");
          if(date_products_map[d] === undefined){
            date_products_map[d] = {date:d, product_versions:[]};
          }
          ver.product = product
          date_products_map[d].product_versions.push(ver);
        }
      });
    });
    _this.dates_products = [];
    Object.keys(date_products_map).forEach(function(d){
      _this.dates_products.push(date_products_map[d]);
    })
    _this.dates_products.sort(function(e1, e2){
      if(e1.date > e2.date){
        return -1;
      }else if(e1.date < e2.date){
        return 1;
      }else{
        return 0;
      }
    });
  };

  _this.render = function(){
    var tmpl = $("#new_products_template").html();
    var compiled_tmpl = _.template(tmpl);
    $("#new_products").html(compiled_tmpl({dates_products:_this.dates_products}));
  };

  return _this;
})();

var eols_box = (function(){
  "use strict";
  var eols_box = {};
  var _this = eols_box;

  _this.show = function(){
    var products_req = $.post($('#sidfm_internal_api_info_recent_product_eols_path').val(), function(res){_this.product_eols = res.product_eols});
    $.when(products_req).done(function(){
      _this.make_data();
      _this.render();
    });
  };

  _this.make_data = function(){
    var now = new Date();
    var dates_eols_map = {};
    _this.product_eols.forEach(function(eol){
      var d = eol.date.replace(/-/g, "/");
      if(dates_eols_map[d] === undefined){
        dates_eols_map[d] = {date:d, dt:new Date(d), eols:[]}
      }
      dates_eols_map[d].eols.push(eol);
    });
    _this.dates_eols = [];
    Object.keys(dates_eols_map).forEach(function(d){
      _this.dates_eols.push(dates_eols_map[d]);
    });
  };

  _this.render = function(){
    var tmpl = $("#recent_eols_template").html();
    var compiled_tmpl = _.template(tmpl);
    var now = new Date();
    $("#recent_eols").html(compiled_tmpl({dates_eols:_this.dates_eols, now:now}));
  };

  return _this;
})();

$(function(){
  "use strict";
  finder_box.init();
  new_products_box.show();
  eols_box.show();
});
