# -*- coding: utf-8 -*-
# Coyright (C) 2020 SofTek Systems, Inc. All rights reserved.

module SIDfm
  module VMRule

    # アクション
    class Action

      @@sync_data = []

      def self.sync_data
        log_debug("vm_rule_action: sync_data(pass1) : #{@@sync_data.size} : #{Time.current}")
        SidfmdVulView.update_from_rule_action(@@sync_data)
        log_debug("vm_rule_action: sync_data(pass2) : #{@@sync_data.size} : #{Time.current}")
        SidfmdHostView.update_from_rule_action(@@sync_data)
        log_debug("vm_rule_action: sync_data(pass3) : #{@@sync_data.size} : #{Time.current}")
        @@sync_data.clear
      end

      def self.add_sync_due_dates(sidfmd_issue_id, html_id, sidfmd_host_id, due_date)
        row = @@sync_data.find {|x|
          x[:sidfmd_issue_id] == sidfmd_issue_id &&
          x[:html_id] == html_id &&
          x[:sidfmd_host_id] == sidfmd_host_id }
        if row then
          row[:due_date] = due_date
        else
          @@sync_data << { :sidfmd_issue_id => sidfmd_issue_id, :html_id => html_id, :sidfmd_host_id => sidfmd_host_id, :due_date => due_date }
        end
      end

      def self.add_sync_status_ids(sidfmd_issue_id, html_id, sidfmd_host_id, status_id)
        row = @@sync_data.find {|x|
          x[:sidfmd_issue_id] == sidfmd_issue_id &&
          x[:html_id] == html_id &&
          x[:sidfmd_host_id] == sidfmd_host_id }
        if row then
          row[:status_id] = status_id
        else
          @@sync_data << { :sidfmd_issue_id => sidfmd_issue_id, :html_id => html_id, :sidfmd_host_id => sidfmd_host_id, :status_id => status_id }
        end
      end

      def self.log_info(*s)
        SIDfm::VMRule.log_info(*s)
      end

      def self.log_error(*s)
        SIDfm::VMRule.log_error(*s)
      end

      def self.log_debug(*s)
        SIDfm::VMRule.log_debug(*s)
      end

      private

      def initialize(rule, args = {})
        raise ArgumentError unless rule
        @rule = rule
        @args = args


        # argsに含まれるホスト一覧や脆弱性一覧
        # filtered_は条件でフィルタされたもの
        # hosts, vulsは定期実行、vulsyncは脆弱性同期時のみであることに注意。
        @prev_vuls_content = (@args[:prev_vuls_content] || [])
        @sync_hostids = (@args[:sync_hostids] || [])
        self.class.log_debug("@sync_hostids:", @sync_hostids)
        @hosts_content = []
        @vuls_content = []
        @vulsync_content = { :add => [], :modify => [], :remove => [] }
        @filtered_hosts_content = []
        @filtered_vuls_content = []
        @filtered_vulsync_content = { :add => [], :modify => [], :remove => [] }

        # ホスト一覧
        SIDfm::VMRule::HOSTS_COTENT_KEYS.each do |key|
          filtered_key = (SIDfm::VMRule::P_FIL + key.to_s).to_sym
          if @args.has_key?(filtered_key) and @args[filtered_key].respond_to?(:'[]') then
            @filtered_hosts_content = @args[filtered_key]
            @filtered_hosts_content.sort! {|x, y| x[:hostname] <=> y[:hostname] }
          end
           if @args.has_key?(key) and @args[key].respond_to?(:'[]') then
             @hosts_content = @args[key]
             @hosts_content.sort! {|x, y| x[:hostname] <=> y[:hostname] }
           end
        end

        # 脆弱性一覧
        SIDfm::VMRule::VULS_CONTENT_KEYS.each do |key|
          filtered_key = (SIDfm::VMRule::P_FIL + key.to_s).to_sym
          if @args.has_key?(filtered_key) and @args[filtered_key].respond_to?(:'[]') then
            @filtered_vuls_content = @args[filtered_key]
            @filtered_vuls_content.sort! {|x, y|
              y[:html_id] <=> x[:html_id]
            }
          end
          if @args.has_key?(key) and @args[key].respond_to?(:'[]') then
            @vuls_content = @args[key]
            @vuls_content.sort! {|x, y|
              y[:html_id] <=> x[:html_id]
            }
          end
        end

        # 脆弱性同期時の追加、更新、削除
        SIDfm::VMRule::VULSYNC_CONTENT_KEYS.each do |key|
          kind = key.to_s.gsub(/^content_/, '').to_sym
          filtered_key = (SIDfm::VMRule::P_FIL + key.to_s).to_sym
          if @args.has_key?(filtered_key) and @args[filtered_key].respond_to?(:'[]') then
            @filtered_vulsync_content[kind] = @args[filtered_key]
            @filtered_vulsync_content[kind].sort! {|x, y|
              y[:html_id] <=> x[:html_id]
            }
          end
          self.class.log_debug("filtered_vulsync_content[#{kind}]=#{@filtered_vulsync_content[kind].size},#{@filtered_vulsync_content[kind].first.inspect}")
          if @args.has_key?(key) and @args[key].respond_to?(:'[]') then
            @vulsync_content[kind] = @args[key]
            @vulsync_content[kind].sort! {|x, y|
              y[:html_id] <=> x[:html_id]
            }
            self.class.log_debug("vulsync_content[#{kind}]=#{@vulsync_content[kind].size},#{@vulsync_content[kind].first.inspect}")
          end
        end
      end

      # コンテンツの差分更新
      # 主にメール送信のアクションで利用できるようにするため。
      # なお、再条件チェックは行わないので、ステータス変更による条件から外れても
      # コンテンツにはデータは含まれたまま。
      # また、@*hosts_contentには反映されない
      def sub_update_content(htmlids)
        if htmlids and htmlids.size > 0 then
          new_content = SidfmdIssue.create_all_content(:html_ids => htmlids, :category_id => 0, :role_admin => true, :no_cache => true, :corresp_required => Sidfm.corresp_required)
          [ @vuls_content, @filtered_vuls_content,
            @vulsync_content[:add], @filtered_vulsync_content[:add],
            @vulsync_content[:modify], @filtered_vulsync_content[:modify],
            @vulsync_content[:remove], @filtered_vulsync_content[:remove], ].each_with_index do |content|
            content.each_with_index do |row, idx|
              if htmlids.include?(row[:html_id]) then
                new_row = new_content.find {|x| row[:html_id] == x[:html_id] }
                if new_row then
                  content[idx] = new_row
                end
              end
            end
          end
          #@@sync_htmlids += htmlids
        end
      end

      # 無効なaction
      def action_invalid(actions)
        actions.each do |action|
          self.class.log_debug("action_invalid:", action)
          action.log_err("無効な操作")
        end
        false
      end

      NOTE_DUEDATE_CHANGE = "ルール「@RULE@」により、対象の脆弱性について対応予定日を@DUEDATE@に変更しました。".freeze
      def sub_note_duedate_change(new_duedate)
        ret = nil
        if new_duedate and new_duedate.respond_to?(:strftime) then
          ret = NOTE_DUEDATE_CHANGE.gsub(/@RULE@/, @rule[:name]).gsub(/@DUEDATE@/, new_duedate.strftime("%Y/%m/%d"))
        end
        ret
      end

      def action_duedate_change(actions)
        ret = false
        actions.each do |action|
          self.class.log_debug("action_duedate_change")
          action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['due-date-change']} 処理開始")
          target = action.target
          proc_empty = action.proc_empty
          detail = action.detail
          permit_update = detail.permit_update
          fixed_date = detail.fixed_date
          refdt_days = detail.refdt_days
          self.class.log_debug("permit_update:", permit_update)
          self.class.log_debug("fixed_date:", fixed_date)
          self.class.log_debug("refdt_days:", refdt_days)
          if SidfmdRuleAction::TARGET_HOSTS == target then
            # ホスト一覧の場合は処理を実行しない
            self.class.log_debug("last resort 1")
            action.log_warn("対象がホスト一覧の場合、処理は実行されません。")
          else
            if SidfmdRuleAction::TARGET_CHANGE then
              self.class.log_debug("target: vulsync all")
              # ADD/MODIFY/REMOVEの全件が対象。
              target_content = @filtered_vulsync_content[:add] + @filtered_vulsync_content[:modify] + @filtered_vulsync_content[:remove]
              target_content.compact!
              target_content.uniq!
              if proc_empty or target_content.size > 0 then
                target_html_ids = target_content.collect {|x| x[:html_id] }.compact.uniq
                begin
                  SidfmdIssue.transaction do
                    query = SidfmdIssue.lock.
                      joins(:issue).
                      where(:html_id => target_html_ids)
                    unless permit_update then
                      query = query.where(:issues => { :due_date => nil })
                    end
                    if @args[:host_ids] then
                      query = query.where(:sidfmd_host_id => @args[:host_ids])
                    end
                    due_date = fixed_date ? fixed_date : Time.current.ago(-refdt_days.days).to_date
                    note = sub_note_duedate_change(due_date)
                    if note then
                      query.each do |i|
                        check_row = target_content.find {|x| x[:html_id] == i.html_id }
                        if check_row and check_row[:sri_cvss_status_ids] and check_row[:sri_cvss_status_ids].find {|x| x[:sidfmd_host_id] == i.sidfmd_host_id } then
                          self.class.log_debug(" change: sidfmd_issue_id=#{i[:id]}, html_id=#{i[:html_id]}, sidfmd_host_id=#{i[:sidfmd_host_id]}, due_date=#{due_date.strftime("%Y/%m/%d")}, note=#{note}")
                          i.change_due_date_with_journal(SidfmdIssue::ADMIN_USER_ID, due_date, note)
                          self.class.add_sync_due_dates(i.id, i.html_id, i.sidfmd_host_id, due_date)
                        else
                          self.class.log_debug(" not change: sidfmd_issue_id=#{i[:id]}, html_id=#{i[:html_id]}, sidfmd_host_id=#{i[:sidfmd_host_id]}, due_date=#{due_date.strftime("%Y/%m/%d")}, note=#{note}")
                        end
                      end
                    else
                      action.log_err("不正な対応予定日が指定されています。")
                      raise StandardError
                    end
                  end
                  sub_update_content(target_html_ids)
                rescue Exception => e
                  self.class.log_error(e)
                end
              else
                # proc_empty=falseでデータが0件の場合は処理はno-op
                self.class.log_debug("last resort 2")
                action.log_info("対象データが0件でデータが無い場合、処理を実行しない指定が行われているため、処理は実行されません。")
              end
            else
              target_content = case target
                               when SidfmdRuleAction::TARGET_VULS
                               then self.class.log_debug("target: vuls(maybe invalid)") ; []
                               when SidfmdRuleAction::TARGET_ADD
                               then self.class.log_debug("target: add") ; @filtered_vulsync_content[:add]
                               when SidfmdRuleAction::TARGET_MODIFY
                               then self.class.log_debug("target: modify") ; @filtered_vulsync_content[:modify]
                               when SidfmdRuleAction::TARGET_REMOVE
                               then self.class.log_debug("target: remove") ; @filtered_vulsync_content[:remove]
                               else
                                 []
                               end
              if proc_emtpy or target_content.size > 0 then
                target_html_ids = target_content.collect {|x| x[:html_id] }.compact.uniq
                begin
                  SidfmdIssue.transaction do
                    query = SidfmdIssue.lock.
                      joins(:issue).
                      where(:html_id => target_html_ids)
                    unless permit_update then
                      query = query.where(:issues => { :due_date => nil })
                    end
                    if @args[:host_ids] then
                      query = query.where(:sidfmd_host_id => @args[:host_ids])
                    end
                    due_date = fixed_date ? fixed_date : Time.current.ago(-refdt_days.days).to_date
                    note = sub_note_duedate_change(due_date)
                    if note then
                      query.each do |i|
                        check_row = target_content.find {|x| x[:html_id] == i.html_id }
                        if check_row and check_row[:sri_cvss_status_ids] and check_row[:sri_cvss_status_ids].find {|x| x[:sidfmd_host_id] == i.sidfmd_host_id } then
                          self.class.log_debug(" change: sidfmd_issue_id=#{i[:id]}, html_id=#{i[:html_id]}, sidfmd_host_id=#{i[:sidfmd_host_id]}, due_date=#{due_date.strftime("%Y/%m/%d")}, note=#{note}")
                          i.change_due_date_with_journal(SidfmdIssue::ADMIN_USER_ID, due_date, note)
                          self.class.add_sync_due_dates(i.id, i.html_id, i.sidfmd_host_id, due_date)
                        else
                          self.class.log_debug(" not change(not match host): sidfmd_issue_id=#{i[:id]}, html_id=#{i[:html_id]}, sidfmd_host_id=#{i[:sidfmd_host_id]}, due_date=#{due_date.strftime("%Y/%m/%d")}, note=#{note}")
                        end
                      end
                    else
                      action.log_err("不正な対応予定日が指定されています。")
                      raise StandardError
                    end
                  end
                  sub_update_content(target_html_ids)
                rescue Exception => e
                  self.class.log_error(e)
                end
              else
                # proc_empty=falseでデータが0件の場合は処理はno-op
                self.class.log_debug("last resort 3")
                action.log_info("対象データが0件でデータが無い場合、処理を実行しない指定が行われているため、処理は実行されません。")
              end
            end
          end
          action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['due-date-change']} 処理終了")
        end
        ret
      end

      # action_status_changeでJournalに書き込むメッセージの処理
      NOTE_STATUS_CHANGE = "ルール「@RULE@」により、対象の脆弱性についてステータスを「@STATUS@」に変更しました。".freeze
      def sub_note_status_change(new_status_id)
        ret = nil
        begin
          issue_status = IssueStatus.find(new_status_id)
          if issue_status then
            ret = NOTE_STATUS_CHANGE.gsub(/@RULE@/, @rule[:name]).gsub(/@STATUS@/, issue_status[:name])
          end
        rescue Exception
        end
        ret
      end

      def action_status_change(actions)
        ret = false
        actions.each do |action|
          self.class.log_debug("action_status_change")
          action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['status-change']} 処理開始")
          target = action.target
          proc_empty = action.proc_empty
          detail = action.detail
          prev_status1 = detail.prev_status1
          prev_status2 = detail.prev_status2
          prev_status = []
          prev_status << prev_status1 if prev_status1
          prev_status << prev_status2 if prev_status2
          prev_status = SidfmdIssue::STATUS_ALL if prev_status.size == 0
          status_id = detail.status_id
          self.class.log_debug("prev_status:", prev_status)
          self.class.log_debug("statud_id:", status_id)
          if SidfmdRuleAction::TARGET_HOSTS == target then
            # ホスト一覧の場合は処理を実行しない
            self.class.log_debug("last resort 1")
            action.log_warn("対象がホスト一覧の場合、処理は実行されません。")
          else
            if target == SidfmdRuleAction::TARGET_CHANGE then
              self.class.log_debug("target: vulsync all")
              self.class.log_debug("proc_empty: #{proc_empty}")
              # ADD/MODIFY/REMOVEの全件が対象。
              target_content = @filtered_vulsync_content[:add] + @filtered_vulsync_content[:modify] + @filtered_vulsync_content[:remove]
              target_content.compact!
              target_content.uniq!
              if proc_empty or target_content.size > 0 then
                target_html_ids = target_content.collect {|x| x[:html_id] }.compact.uniq
                self.class.log_debug("target_html_ids: #{target_html_ids.size}")
                begin
                  SidfmdIssue.transaction do
                    query = SidfmdIssue.lock.
                      joins(:issue).
                      where(:html_id => target_html_ids).
                      where(:issues => { :status_id => prev_status })
                    if @args[:host_ids] then
                      query = query.where(:sidfmd_host_id => @args[:host_ids])
                    end
                    note = sub_note_status_change(status_id)
                    self.class.log_debug("note: #{note}")
                    if note then
                      query.each do |i|
                        check_row = target_content.find {|x| x[:html_id] == i.html_id }
                        if check_row and check_row[:sri_cvss_status_ids] and check_row[:sri_cvss_status_ids].find {|x| x[:sidfmd_host_id] == i.sidfmd_host_id } then
                          self.class.log_debug(" change: sidfmd_issue_id=#{i[:id]}, html_id=#{i[:html_id]}, sidfmd_host_id=#{i[:sidfmd_host_id]}, status_id=#{status_id}, note=#{note}")
                          i.change_status_with_journal(SidfmdIssue::ADMIN_USER_ID, status_id, note)
                          self.class.add_sync_status_ids(i.id, i.html_id, i.sidfmd_host_id, status_id)
                        else
                          self.class.log_debug(" not change(not match host): sidfmd_issue_id=#{i[:id]}, html_id=#{i[:html_id]}, sidfmd_host_id=#{i[:sidfmd_host_id]}, status_id=#{status_id}, note=#{note}")
                        end
                      end
                    else
                      action.log_err("不正な対応ステータスが指定されています。")
                      raise StandardError
                    end
                  end
                  sub_update_content(target_html_ids)
                rescue Exception => e
                  self.class.log_error(e)
                end
              else
                # proc_empty=falseでデータが0件の場合は処理はno-op
                self.class.log_debug("last resort 2") 
                action.log_info("対象データが0件でデータが無い場合、処理を実行しない指定が行われているため、処理は実行されません。")
              end
            else
              target_content = case target
                               when SidfmdRuleAction::TARGET_VULS
                               then self.class.log_debug("target: vuls(maybe invalid)") ; []
                               when SidfmdRuleAction::TARGET_ADD
                               then self.class.log_debug("target: add") ; @filtered_vulsync_content[:add]
                               when SidfmdRuleAction::TARGET_MODIFY
                               then self.class.log_debug("target: modify") ; @filtered_vulsync_content[:modify]
                               when SidfmdRuleAction::TARGET_REMOVE
                               then self.class.log_debug("target: remove") ; @filtered_vulsync_content[:remove]
                               else
                                 []
                               end
              if proc_empty or target_content.size > 0 then
                target_html_ids = target_content.collect {|x| x[:html_id] }.compact.uniq
                begin
                  SidfmdIssue.transaction do
                    query = SidfmdIssue.lock.
                      joins(:issue).
                      where(:html_id => target_html_ids).
                      where(:issues => { :status_id => prev_status })
                    if @args[:host_ids] then
                      query = query.where(:sidfmd_host_id => @args[:host_ids])
                    end
                    note = sub_note_status_change(status_id)
                    if note then
                      query.each do |i|
                        check_row = target_content.find {|x| x[:html_id] == i.html_id }
                        if check_row and check_row[:sri_cvss_status_ids] and check_row[:sri_cvss_status_ids].find {|x| x[:sidfmd_host_id] == i.sidfmd_host_id } then
                          self.class.log_debug(" change: sidfmd_issue_id=#{i[:id]}, html_id=#{i[:html_id]}, sidfmd_host_id=#{i[:sidfmd_host_id]}, status_id=#{status_id}, note=#{note}")

                          i.change_status_with_journal(SidfmdIssue::ADMIN_USER_ID, status_id, note)
                          self.class.add_sync_status_ids(i.id, i.html_id, i.sidfmd_host_id, status_id)
                        else
                          self.class.log_debug(" not change(not match host): sidfmd_issue_id=#{i[:id]}, html_id=#{i[:html_id]}, sidfmd_host_id=#{i[:sidfmd_host_id]}, status_id=#{status_id}, note=#{note}")
                        end
                      end
                    else
                      action.log_err("不正な対応ステータスが指定されています。")
                      raise StandardError
                    end
                  end
                  sub_update_content(target_html_ids)
                rescue Exception => e
                  self.class.log_error(e)
                end
              else
                # proc_empty=falseでデータが0件の場合は処理はno-op
                self.class.log_debug("last resort 3")
                action.log_info("対象データが0件でデータが無い場合、処理を実行しない指定が行われているため、処理は実行されません。")
              end
            end
          end
          action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['status-change']} 処理終了")
        end
        ret
      end

      def action_mail_user(actions)
        # SidfmdRuleAMailUser
        target_vuls = SidfmdRuleAction::TARGET_VULS
        target_hosts = SidfmdRuleAction::TARGET_HOSTS
        target_add = SidfmdRuleAction::TARGET_ADD
        target_modify = SidfmdRuleAction::TARGET_MODIFY
        target_remove = SidfmdRuleAction::TARGET_REMOVE
        data_for_keyword = {}
        data_for_keyword[:hosts_content] = @hosts_content
        data_for_keyword[:vuls_content] = @vuls_content
        data_for_keyword[:filtered_hosts_content] = @filtered_hosts_content
        data_for_keyword[:filtered_vuls_content] = @filtered_vuls_content
        data_for_keyword[:vulsync_content] = @vulsync_content
        data_for_keyword[:filtered_vulsync_content] = @filtered_vulsync_content
        data_for_keyword[:sync_hostids] = @sync_hostids
    
        user_ids = []
        user_emails = []
        if actions.first then
          actions.first.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['send-mail-user']} 処理開始")
        end
        actions.each do |action|
          self.class.log_debug("action_mail_user")
          detail = action.detail
          action_target = action.target
          proc_empty = action.proc_empty
          case action_target
          when target_vuls then
            if @filtered_vuls_content.empty? && !proc_empty then
              next
            end
          when target_hosts then
            if @filtered_hosts_content.empty? && !proc_empty then
              next
            end
          else
            proc_content_size = case action_target
                                when target_add then @filtered_vulsync_content[:add].size
                                when target_modify then @filtered_vulsync_content[:modify].size
                                when target_remove then @filtered_vulsync_content[:remove].size
                                else @filtered_vulsync_content[:add].size +
                                     @filtered_vulsync_content[:modify].size +
                                     @filtered_vulsync_content[:remove].size
                                end
            if proc_content_size == 0 && !proc_empty then
              next
            end
          end
          user_ids << detail.user_id if detail.user_id
          user_emails << detail.email if detail.email
        end
        if user_ids.size > 0 || user_emails.size > 0 then
          detail = actions[0].detail
          if user_ids.size > 0 then
            addresses = User.where(:id => user_ids).map{|user| 
              if user.mail_notification == "only_my_events" then
                %("#{user.lastname} #{user.firstname}" <#{user.mail}>)
              end
            }.compact
          else
            addresses = user_emails
          end
          if detail.from_user_id then
            sender = User.find(detail.from_user_id).mail
          else
            sender = detail.from_email
          end
          subject = detail.subject
          body = detail.body
          subjects_keywords = SidfmdMailer.get_keywords(subject)
          body_keywords = SidfmdMailer.get_keywords(body)
          keywords = body_keywords.merge(subjects_keywords)
          keywords_with_data = SidfmdMailer.assign_data_to_keywords(keywords, data_for_keyword, @rule)
          mail_subject = SidfmdMailer.replace_keywords(subject, keywords_with_data)
          mail_body = SidfmdMailer.replace_keywords(body, keywords_with_data)
          mail = {}
          mail[:from] = sender
          mail[:subject] = mail_subject
          mail[:message] = mail_body
          send_mail_num = SidfmdMailer::SEND_MAIL_NUM
          time_out_second = SidfmdMailer::SEND_MAIL_TIMEOUT_SECOND
          while (send_mail_targets = addresses.shift(send_mail_num)).size > 0 do
            threads = send_mail_targets.map{|fullname_and_address|
              Thread.new(mail.deep_dup, fullname_and_address){|cp_mail, data|
                ret = {}
                cp_mail[:to] = data
                begin
                  SidfmdMailer.send_mail(cp_mail).deliver_now
                rescue Exception => e
                  ret[:exception] = e
                end
                ret
              }
            }
            threads.each do |result|
              result = result.join(time_out_second)
              if result then
                result = result.value
                if result[:exception] then Rails.logger.error("failed send mail. #{result[:exception]}") end
              else
                Rails.logger.error("time-out of sending a mail")
              end
            end  
          end
        end
        if actions.first then
          actions.first.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['send-mail-user']} 処理終了")
        end
        return true
      end

      def action_mail_category(actions)
        # SidfmdRuleAMailCategory
        target_vuls = SidfmdRuleAction::TARGET_VULS
        target_hosts = SidfmdRuleAction::TARGET_HOSTS
        target_add = SidfmdRuleAction::TARGET_ADD
        target_modify = SidfmdRuleAction::TARGET_MODIFY
        target_remove = SidfmdRuleAction::TARGET_REMOVE
        data_for_keyword = {}
        data_for_keyword[:hosts_content] = @hosts_content
        data_for_keyword[:vuls_content] = @vuls_content
        data_for_keyword[:filtered_hosts_content] = @filtered_hosts_content
        data_for_keyword[:filtered_vuls_content] = @filtered_vuls_content
        data_for_keyword[:vulsync_content] = @vulsync_content
        data_for_keyword[:filtered_vulsync_content] = @filtered_vulsync_content
        data_for_keyword[:sync_hostids] = @sync_hostids

        actions.each do |action|
          detail = action.detail
          self.class.log_debug("action_mail_category")
          action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['send-mail-category']} 処理開始")
          action_target = action.target
          proc_empty = action.proc_empty
          case action_target 
          when target_vuls then
            if @filtered_vuls_content.empty? && !proc_empty then
              action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['send-mail-category']} 処理終了")
              next
            end
          when target_hosts then
            if @filtered_hosts_content.empty? && !proc_empty then
              action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['send-mail-category']} 処理終了")
              next 
            end
          else
            proc_content_size = case action_target
                                  when target_add then @filtered_vulsync_content[:add].size
                                  when target_modify then @filtered_vulsync_content[:modify].size
                                  when target_remove then @filtered_vulsync_content[:remove].size
                                  else @filtered_vulsync_content[:add].size + 
                                       @filtered_vulsync_content[:modify].size + 
                                       @filtered_vulsync_content[:remove].size
                                end
            if proc_content_size == 0 && !proc_empty then
              action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['send-mail-category']} 処理終了")
              next
            #else
            #  data_for_keyword[:vulsync_content] = @vulsync_content
            #  data_for_keyword[:filtered_vulsync_content] = @filtered_vulsync_content
            end
          end
          c_category_id = detail.sidfmd_category_id
          if c_category_id then
            user_ids = SidfmdCategoriesUser.where(:sidfmd_category_id => c_category_id).map(&:user_id)
          else
            user_ids = SidfmdCategoriesUser.all.map(&:user_id)
          end
          address_users = User.where(:id => user_ids).map{|user| 
            if user.mail_notification == "only_my_events" then
              %("#{user.lastname} #{user.firstname}" <#{user.mail}>)
            end
          }.compact
          if detail.from_user_id then
            sender_address = User.find(detail.from_user_id).mail
          else
            sender_address = detail.from_email
          end           
          subject = detail.subject
          body = detail.body
          subjects_keywords = SidfmdMailer.get_keywords(subject)
          body_keywords = SidfmdMailer.get_keywords(body)
          keywords = body_keywords.merge(subjects_keywords)
          keywords_with_data = SidfmdMailer.assign_data_to_keywords(keywords, data_for_keyword, @rule) 
          mail_subject = SidfmdMailer.replace_keywords(subject, keywords_with_data)
          mail_body = SidfmdMailer.replace_keywords(body, keywords_with_data)
          mail = {}
          mail[:from] = sender_address
          mail[:subject] = mail_subject
          mail[:message] = mail_body
          send_mail_num = SidfmdMailer::SEND_MAIL_NUM
          time_out_second = SidfmdMailer::SEND_MAIL_TIMEOUT_SECOND
          while (send_mail_targets = address_users.shift(send_mail_num)).size > 0 do
            threads = send_mail_targets.map{|fullname_and_address|
              Thread.new(mail.deep_dup, fullname_and_address){|cp_mail, data|
                ret = {}
                cp_mail[:to] = data
                begin
                  SidfmdMailer.send_mail(cp_mail).deliver_now
                rescue Exception => e
                  ret[:exception] = e
                end
                ret
              }
            }
            threads.each do |result|
              result = result.join(time_out_second)
              if result then 
                result = result.value 
                if result[:exception] then Rails.logger.error("failed send mail. #{result[:exception]}") end
              else
                Rails.logger.error("time-out of sending a mail")
              end
            end 
          end
          action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['send-mail-category']} 処理終了")
        end
        return true
      end

      def action_send_syslog(actions)
        actions.each do |action|
          self.class.log_debug("action_send_syslog")
          action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['send-syslog']} 処理開始")
          action_target = action.target
          proc_empty = action.proc_empty
          detail = action.detail
          protocol = detail.protocol
          address = SidfmdRuleASendSyslog.get_address(detail.target)
          port = detail.port
          hostname = detail.hostname
          ident = detail.ident
          facility = detail.facility
          severity = detail.severity
          message = detail.message
          data_for_keyword = {}
          data_for_keyword[:hosts_content] = @hosts_content
          data_for_keyword[:vuls_content] = @vuls_content
          data_for_keyword[:filtered_hosts_content] = @filtered_hosts_content
          data_for_keyword[:filtered_vuls_content] = @filtered_vuls_content
          case action_target
          when SidfmdRuleAction::TARGET_VULS then
            if @filtered_vuls_content.empty? && !proc_empty then
              self.class.log_debug("条件でフィルタ後の脆弱性一覧が0件のため、処理は実行されません。")
              action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['send-syslog']} 処理終了")
              next
            end
          when SidfmdRuleAction::TARGET_HOSTS then
            if @filtered_hosts_content.empty? && !proc_empty then
              action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['send-syslog']} 処理終了")
              next
            end
          else
            proc_content_size = case action_target
                                when SidfmdRuleAction::TARGET_ADD then @filtered_vulsync_content[:add].size
                                when SidfmdRuleAction::TARGET_MODIFY then @filtered_vulsync_content[:modify].size
                                when SidfmdRuleAction::TARGET_REMOVE then @filtered_vulsync_content[:remove].size
                                else @filtered_vulsync_content[:add].size +
                                     @filtered_vulsync_content[:modify].size +
                                     @filtered_vulsync_content[:remove].size
                                end
            if proc_content_size == 0 && !proc_empty then
              action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['send-syslog']} 処理終了")
              next
            else
              data_for_keyword[:vulsync_content] = @vulsync_content
              data_for_keyword[:filtered_vulsync_content] = @filtered_vulsync_content
            end
          end
          begin
            keywords = SidfmdMailer.get_keywords(message)
            keywords_with_data = SidfmdMailer.assign_data_to_keywords(keywords, data_for_keyword, @rule)
            message = SidfmdMailer.replace_keywords(message, keywords_with_data)
            syslog = SyslogRuby::Logger.new(ident, facility.to_sym, uri: "#{protocol}://#{address}:#{port}")
            syslog.hostname = hostname
            syslog.send("#{severity.downcase}", "#{message}")
          rescue Exception => e
            Rails.logger.error(e)
          end
          action.log_info("操作:#{SidfmdRule::RULE_ACTION_NAMES['send-syslog']} 処理終了")
        end
        return true
      end

      public

      def run
        ret = true
        self.class.log_debug("rule = #{@rule.inspect}")
        self.class.log_debug("args = #{@args.keys.inspect}")
        actions = @rule.sidfmd_rule_actions
        duedate_change_actions =[]
        status_change_actions = []
        mail_category_actions = []
        mail_user_actions = []
        send_syslog_actions = []
        invalid_actions = []
        actions.each do |action|
          case action.detail
          when SidfmdRuleADuedateChange then duedate_change_actions << action
          when SidfmdRuleAStatusChange  then status_change_actions << action
          when SidfmdRuleAMailCategory  then mail_category_actions << action
          when SidfmdRuleAMailUser      then mail_user_actions << action
          when SidfmdRuleASendSyslog    then send_syslog_actions << action
          else                               invalid_actions << action
          end
        end
        begin
          if !invalid_actions.empty? then
            ret &= action_invalid(invalid_actions)
          end
          if !duedate_change_actions.empty? then
            ret &= action_duedate_change(duedate_change_actions)
          end
          if !status_change_actions.empty? then
            ret &= action_status_change(status_change_actions)
          end
          if !mail_category_actions.empty? then
            ret &= action_mail_category(mail_category_actions)
          end
          if !mail_user_actions.empty? then
            ret &= action_mail_user(mail_user_actions)
          end
          if !send_syslog_actions.empty? then
            ret &= action_send_syslog(send_syslog_actions)
          end
        rescue Exception => e
          self.class.log_error(e)
        end
        ret
      end
    end

  end
end
