# -*- coding: utf-8 -*-

# Copyright (C) 2020 SofTek Systems, Inc. All rights reserved.

module SIDfm
  module VMRule
    class Trigger

      private

      def initialize(args = {})
        @args = args
      end

      def run_invalid
        @args[:trigger] = :invalid
        []
      end

      def run_periodic
        @args[:trigger] = :periodic
        ret = SidfmdRule.match_rule_ids(periodic: true)
        ret.each do |rule_id|
          SidfmdRule.transaction do
            rule = SidfmdRule.lock.find(rule_id)
            if rule then
              trigger = rule.sidfmd_rule_trigger
              if trigger then
                trigger.log_info("タイミング適合（定期実行）")
              end
            end
          end
        end
        ret
      end

      def run_vulsync
        @args[:trigger] = :vulsync
        ret = SidfmdRule.match_rule_ids(vulsync: true)
        ret.each do |rule_id|
          rule = SidfmdRule.find(rule_id)
          if rule then
            trigger = rule.sidfmd_rule_trigger
            if trigger then
              trigger.log_info("タイミング適合（脆弱性同期時）")
            end
          end
        end
        ret
      end

      public

      def run(trigger_type)
        case trigger_type
        when :periodic then run_periodic
        when :vulsync  then run_vulsync
        else                run_invalid
        end
      end

    end
  end
end
